package gs.timing.v0

import scala.concurrent.duration.FiniteDuration

/** Represents elapsed, monotonic time, expressed as nanoseconds.
  *
  * @param start
  *   The start time of this duration (an absolute monotonic value).
  * @param end
  *   The end time of this duration (an absolute monotonic value).
  * @param duration
  *   The duration that elapsed, expressed as nanoseconds.
  */
case class ElapsedTime(
  start: Long,
  end: Long,
  duration: FiniteDuration
):
  /** @return
    *   The elapsed duration, expressed as nanoseconds.
    */
  def toNanoseconds(): Long = duration.toNanos

  /** @return
    *   The elapsed duration, expressed as milliseconds.
    */
  def toMilliseconds(): Long = duration.toMillis

  /** @return
    *   The elapsed duration, expressed as seconds.
    */
  def toSeconds(): Long = duration.toSeconds
